@@ -349,19 +349,25 @@ sub first {
     my $first = $self->new( @{$self->{list}} [0] );
     return $first unless wantarray;
     my $res = $self->new->no_cleanup;
-    push @{$res->{list}}, @{$self->{list}} [1 .. -1];
+    push @{$res->{list}}, @{$self->{list}} [1 .. $#{$self->{list}}];
+    return @{$self->{first}} = ($first) if $res->is_null;
     return @{$self->{first}} = ($first, $res);
 }
 
-# TODO: finish last() && test!
 sub last {
     my $self = shift;
-    return undef unless @{$self->{list}};
+    if (exists $self->{last} ) {
+        return wantarray ? @{$self->{last}} : $self->{last}[0];
+    }
+    unless ( @{$self->{list}} ) {
+        return wantarray ? (undef, 0) : undef;
+    }
     my $last = $self->new( @{$self->{list}} [-1] );
-    return $last if ! wantarray;  # last
+    return $last unless wantarray;  
     my $res = $self->new->no_cleanup;
-    push @{$res->{list}}, @{$self->{list}} [0 .. -2];
-    return $last, $res;
+    push @{$res->{list}}, @{$self->{list}} [0 .. $#{$self->{list}}-1];
+    return @{$self->{last}} = ($last) if $res->is_null;
+    return @{$self->{last}} = ($last, $res);
 }
 
 sub is_null {
@@ -961,7 +967,7 @@ __END__
 
 =head1 INTERNALS
 
-The internal representation of a 'span' is a hash:
+The internal representation of a I<span> is a hash:
 
     { a =>   start of span,
       b =>   end of span,
@@ -983,7 +989,17 @@ is represented by the array of hashes:
         { a => 400, b => infinity, open_begin => 0, open_end => 1 },
     ]
 
-The infinity value is generated by Perl, when it finds a numerical overflow:
+The I<density> of a set is stored in the C<tolerance> variable:
+
+    tolerance => 0;  # the set is made of real numbers.
+
+    tolerance => 1;  # the set is made of integers.
+
+The C<type> variable stores the I<class> of objects that will be stored in the set.
+
+    type => 'DateTime';   # this is a set of DateTime objects
+
+The I<infinity> value is generated by Perl, when it finds a numerical overflow:
 
     $inf = 100**100**100;
 
